﻿
Public Class Form1
    Private TargetDevice As Dataq.Devices.DI2108P.Device
    Private taskRead As Task
    Private cancelRead As CancellationTokenSource
    ' Define class that will print the data to the GUI
    Private progress As IProgress(Of String) =
        New Progress(Of String)(Sub(s As String)
                                    tbOutput.AppendText(s)
                                End Sub)

    Private Async Sub btnState_Click(sender As Object, e As EventArgs) Handles btnState.Click
        btnState.Enabled = False
        If cancelRead IsNot Nothing Then
            'get here if an acquisition process is in progress and we've been commanded to stop
            cancelRead.Cancel() 'cancel the read process
            cancelRead = Nothing
            Await taskRead 'wait for the read process to complete
            taskRead = Nothing
            Await TargetDevice.AcquisitionStopAsync() 'stop the device from acquiring 
            ' Change button text/state
            btnState.Text = "Start"
            GroupBox1.Enabled = True
            GroupBox2.Enabled = True
            tbSampleRate.Enabled = True
            tbRateFilter.Enabled = True
            tbDecimation.Enabled = True
            btnDigOut.Enabled = False

        Else
            'get here if we're starting a new acquisition process
            TargetDevice.Channels.Clear() 'initialize the device
            ConfigureAnalogChannels()
            ConfigureDigitalChannels()
            Try
                Await TargetDevice.InitializeAsync() 'configure the device as defined. Errors if no channels are enabled
            Catch ex As Exception
                'Detect if no channels are enabled, and bail if so. 
                MessageBox.Show("Please enable at least one analog channel or digital port.",
                                "No Enabled Channels", MessageBoxButtons.OK, MessageBoxIcon.Error)
                btnState.Enabled = True
                Exit Sub
            End Try

            If SampleRateBad() Then
                'get here if requested sample rate is out of range
                'It's a bust, so...
                btnState.Enabled = True
                Exit Sub
            End If
            'otherwise, the selected sample rate is good, so use it
            TargetDevice.SampleRate = tbSampleRate.Text
            Await TargetDevice.InitializeAsync() 'configure the device again with a known good sample rate

            'now determine what sample rate per channel the device is using from the 
            'first enabled input channel, and display it
            Dim FirstInChannel As Dataq.Devices.DI2108P.ChannelIn
            Dim NoInputChannels As Boolean = True
            For index = 0 To TargetDevice.Channels.Count - 1
                If TypeOf TargetDevice.Channels(index) Is Dataq.Devices.IChannelIn Then
                    FirstInChannel = TargetDevice.Channels(index)
                    lblActual.Text = Math.Round(FirstInChannel.GetSampleRate(TargetDevice), 2)
                    NoInputChannels = False
                    Exit For
                End If
            Next
            If NoInputChannels Then
                MessageBox.Show("Please configure at least one analog channel or digital port as an input",
                                "No Inputs Enabled", MessageBoxButtons.OK, MessageBoxIcon.Error)
                btnState.Enabled = True
                Exit Sub
            End If
            'Everything is good, so...
            btnState.Text = "Stop" 'change button text to "Stop" from "Start"
            cancelRead = New CancellationTokenSource() ' Create the cancellation token
            Await TargetDevice.AcquisitionStartAsync() 'start acquiring

            ' NOTE: assumes at least one input channel enabled
            ' Start a task in the background to read data
            taskRead = Task.Run(Async Function()
                                    'capture the first channel programmed as an input (MasterChannel)
                                    'and use it to track data availability for all input channels
                                    Dim MasterChannel As Dataq.Devices.IChannelIn = Nothing
                                    For index = 0 To TargetDevice.Channels.Count
                                        If TypeOf TargetDevice.Channels(index) Is Dataq.Devices.IChannelIn Then
                                            MasterChannel = TargetDevice.Channels(index) ' we have our channel 
                                            Exit For
                                        End If
                                    Next

                                    ' Keep reading while acquiring data
                                    While TargetDevice.IsAcquiring
                                        ' Read data and catch if cancelled (to exit loop and continue)
                                        Try
                                            'throws an error if acquisition has been cancelled
                                            'otherwise refreshes the buffer DataIn with new data
                                            Await TargetDevice.ReadDataAsync(cancelRead.Token)
                                        Catch ex As OperationCanceledException
                                            'get here if acquisition cancelled
                                            Exit While
                                        End Try
                                        'get here if acquisition is still active
                                        If MasterChannel.DataIn.Count = 0 Then
                                            'get here if no data in the channel buffer
                                            Continue While
                                        End If

                                        ' We have data. Convert it to strings
                                        Dim temp As String = ""
                                        Dim temp1 As String = ""
                                        ' NOTE: assuming all input channels contain exact same amount of data
                                        'Think of data for each input channel organized in columns, 
                                        'one column per channel (DataIn (index)). Rows represent scans.
                                        'Data is extracted by the following routine 
                                        For index = 0 To MasterChannel.DataIn.Count - 1 ' this is the row (scan) counter
                                            For Each ch In TargetDevice.Channels 'this is the column (channel) counter
                                                'get a channel value and convert it to a string
                                                'format the output string depending upon the type of input
                                                If (TypeOf ch Is Dataq.Devices.IAnalogVoltage) Or
                                                (TypeOf ch Is Dataq.Devices.IFrequency) Then
                                                    temp1 = CType(ch, Dataq.Devices.IChannelIn).DataIn(index).ToString("0.0000")
                                                Else
                                                    'must be either dig in or count so we need just a single integer
                                                    temp1 = CType(ch, Dataq.Devices.IChannelIn).DataIn(index).ToString("0")
                                                End If
                                                temp1 = temp1 + ", "
                                                temp = temp + temp1 'append the channel value to the output

                                            Next ' get the next channel
                                            'get here when a channel row is complete
                                            'strip the trailing comma from the last column position
                                            temp = temp.Substring(0, temp.Length - 2) + Environment.NewLine
                                        Next 'get the next row
                                        ' purge displayed data from the buffer
                                        For Each ch In TargetDevice.Channels
                                            If TypeOf ch Is Dataq.Devices.IChannelIn Then
                                                CType(ch, Dataq.Devices.IChannelIn).DataIn.Clear()
                                            End If
                                        Next
                                        ' Report the data to be printed to the GUI
                                        progress.Report(temp)
                                    End While
                                    progress.Report("Stopped" + Environment.NewLine)
                                End Function, cancelRead.Token)
            GroupBox1.Enabled = False
            GroupBox2.Enabled = False
            tbSampleRate.Enabled = False
            tbRateFilter.Enabled = False
            tbDecimation.Enabled = False
            btnDigOut.Enabled = True
        End If
        ' Enable button once operation has completed
        btnState.Enabled = True
    End Sub

    Function SampleRateBad() As Boolean
        'ensure that the sample rate per channel is not out of range. Use TargetDevice.CountChannelsInScan
        'instead of TargetDevice.Channels.Count, since the latter treats each digital port as a unique channel when
        'in fact digital ins are grouped together and reported as one. Counter and rate inputs are treated as individual channels.
        Dim Desired As Double = tbSampleRate.Text / TargetDevice.CountChannelsInScan
        Dim ActualMin As Double = TargetDevice.SupportedSampleRateRange.Minimum * TargetDevice.CountChannelsInScan
        Dim ActualMax As Double = TargetDevice.SupportedSampleRateRange.Maximum / TargetDevice.CountChannelsInScan

        If (Desired < TargetDevice.SupportedSampleRateRange.Minimum) Or
            (Desired > TargetDevice.SupportedSampleRateRange.Maximum) Then
            ActualMin = Math.Truncate(ActualMin) + 1
            ActualMax = Math.Truncate(ActualMax)
            MessageBox.Show("Selected sample throughput rate is outside the range of " + ActualMin.ToString + " to " + ActualMax.ToString + " Hz inclusive for your current channel settings.", "Sample Rate Error",
                            MessageBoxButtons.OK, MessageBoxIcon.Error)
            Return True
            Exit Function
        End If
        Return False
    End Function

    Sub ConfigureDigitalChannels()
        Dim DigitalInChan As Dataq.Devices.DI2108P.DigitalIn
        Dim DigitalOutChan As Dataq.Devices.DI2108P.DigitalOut
        Dim RateInChan As Dataq.Devices.DI2108P.FrequencyIn
        Dim CountInChan As Dataq.Devices.DI2108P.CounterIn

        'configure digtial port 0
        If cbD0.Checked Then
            Select Case True
                Case cmbD0Mode.Text = "Switch"
                    DigitalOutChan = CType(TargetDevice.ChannelFactory(0, GetType(Dataq.Devices.DI2108P.DigitalOut)),
                        Dataq.Devices.DI2108P.DigitalOut)
                Case Else 'else has to be an input
                    DigitalInChan = CType(TargetDevice.ChannelFactory(0, GetType(Dataq.Devices.DI2108P.DigitalIn)),
                        Dataq.Devices.DI2108P.DigitalIn)
                    DigitalInChan.AcquisitionMode.Samples = CType(tbDecimation.Text, Integer)
            End Select
        End If

        'configure digtial port 1
        If cbD1.Checked Then
            Select Case True
                Case cmbD1Mode.Text = "Switch"
                    DigitalOutChan = CType(TargetDevice.ChannelFactory(1, GetType(Dataq.Devices.DI2108P.DigitalOut)),
                        Dataq.Devices.DI2108P.DigitalOut)
                Case Else 'else has to be an input
                    DigitalInChan = CType(TargetDevice.ChannelFactory(1, GetType(Dataq.Devices.DI2108P.DigitalIn)),
                        Dataq.Devices.DI2108P.DigitalIn)
                    DigitalInChan.AcquisitionMode.Samples = CType(tbDecimation.Text, Integer)
            End Select
        End If

        'configure digtial port 2
        If cbD2.Checked Then
            Select Case True
                Case cmbD2Mode.Text = "Switch"
                    DigitalOutChan = CType(TargetDevice.ChannelFactory(2, GetType(Dataq.Devices.DI2108P.DigitalOut)),
                        Dataq.Devices.DI2108P.DigitalOut)
                Case cmbD2Mode.Text.Contains("Rate")
                    'rate channels are identified in logical order, so this is rate channel 1
                    RateInChan = CType(TargetDevice.ChannelFactory(1, GetType(Dataq.Devices.DI2108P.FrequencyIn)),
                        Dataq.Devices.DI2108P.FrequencyIn)
                    RateInChan.AveragingFactor = CType(tbRateFilter.Text, Integer)
                    RateInChan.AcquisitionMode.Samples = CType(tbDecimation.Text, Integer)
                    Select Case True
                        Case cmbD2Mode.Text.Contains("50 kHz")
                            RateInChan.FrequencyRange.Maximum = 50000
                        Case cmbD2Mode.Text.Contains("20 kHz")
                            RateInChan.FrequencyRange.Maximum = 20000
                        Case cmbD2Mode.Text.Contains("10 kHz")
                            RateInChan.FrequencyRange.Maximum = 10000
                        Case cmbD2Mode.Text.Contains("5 kHz")
                            RateInChan.FrequencyRange.Maximum = 5000
                        Case cmbD2Mode.Text.Contains("2 kHz")
                            RateInChan.FrequencyRange.Maximum = 2000
                        Case cmbD2Mode.Text.Contains("1 kHz")
                            RateInChan.FrequencyRange.Maximum = 1000
                        Case cmbD2Mode.Text.Contains("500 Hz")
                            RateInChan.FrequencyRange.Maximum = 500
                        Case cmbD2Mode.Text.Contains("200 Hz")
                            RateInChan.FrequencyRange.Maximum = 200
                        Case cmbD2Mode.Text.Contains("100 Hz")
                            RateInChan.FrequencyRange.Maximum = 100
                        Case cmbD2Mode.Text.Contains("50 Hz")
                            RateInChan.FrequencyRange.Maximum = 50
                        Case cmbD2Mode.Text.Contains("20 Hz")
                            RateInChan.FrequencyRange.Maximum = 20
                        Case cmbD2Mode.Text.Contains("10 Hz")
                            RateInChan.FrequencyRange.Maximum = 10
                    End Select
                Case Else 'else has to be a discrete input
                    DigitalInChan = CType(TargetDevice.ChannelFactory(2, GetType(Dataq.Devices.DI2108P.DigitalIn)),
                        Dataq.Devices.DI2108P.DigitalIn)
                    DigitalInChan.AcquisitionMode.Samples = CType(tbDecimation.Text, Integer)
            End Select
        End If

        'configure digtial port 3
        If cbD3.Checked Then
            Select Case True
                Case cmbD3Mode.Text = "Switch"
                    DigitalOutChan = CType(TargetDevice.ChannelFactory(3, GetType(Dataq.Devices.DI2108P.DigitalOut)),
                        Dataq.Devices.DI2108P.DigitalOut)
                Case cmbD3Mode.Text = "Count"
                    'count channels are identified in logical order, so this is count channel 1
                    CountInChan = CType(TargetDevice.ChannelFactory(1, GetType(Dataq.Devices.DI2108P.CounterIn)),
                        Dataq.Devices.DI2108P.CounterIn)
                    CountInChan.AcquisitionMode.Samples = CType(tbDecimation.Text, Integer)
                Case Else 'else has to be a discrete input
                    DigitalInChan = CType(TargetDevice.ChannelFactory(3, GetType(Dataq.Devices.DI2108P.DigitalIn)),
                        Dataq.Devices.DI2108P.DigitalIn)
                    DigitalInChan.AcquisitionMode.Samples = CType(tbDecimation.Text, Integer)
            End Select
        End If

        'configure digtial port 4
        If cbD4.Checked Then
            Select Case True
                Case cmbD4Mode.Text = "Switch"
                    DigitalOutChan = CType(TargetDevice.ChannelFactory(4, GetType(Dataq.Devices.DI2108P.DigitalOut)),
                        Dataq.Devices.DI2108P.DigitalOut)
                Case Else 'else has to be an input
                    DigitalInChan = CType(TargetDevice.ChannelFactory(4, GetType(Dataq.Devices.DI2108P.DigitalIn)),
                        Dataq.Devices.DI2108P.DigitalIn)
                    DigitalInChan.AcquisitionMode.Samples = CType(tbDecimation.Text, Integer)
            End Select
        End If

        'configure digtial port 5
        If cbD5.Checked Then
            Select Case True
                Case cmbD5Mode.Text = "Switch"
                    DigitalOutChan = CType(TargetDevice.ChannelFactory(5, GetType(Dataq.Devices.DI2108P.DigitalOut)),
                        Dataq.Devices.DI2108P.DigitalOut)
                Case Else 'else has to be an input
                    DigitalInChan = CType(TargetDevice.ChannelFactory(5, GetType(Dataq.Devices.DI2108P.DigitalIn)),
                        Dataq.Devices.DI2108P.DigitalIn)
                    DigitalInChan.AcquisitionMode.Samples = CType(tbDecimation.Text, Integer)
            End Select
        End If

        'configure digtial port 6
        If cbD6.Checked Then
            Select Case True
                Case cmbD6Mode.Text = "Switch"
                    DigitalOutChan = CType(TargetDevice.ChannelFactory(6, GetType(Dataq.Devices.DI2108P.DigitalOut)),
                        Dataq.Devices.DI2108P.DigitalOut)
                Case Else 'else has to be an input
                    DigitalInChan = CType(TargetDevice.ChannelFactory(6, GetType(Dataq.Devices.DI2108P.DigitalIn)),
                        Dataq.Devices.DI2108P.DigitalIn)
                    DigitalInChan.AcquisitionMode.Samples = CType(tbDecimation.Text, Integer)
            End Select
        End If
    End Sub

    Sub ConfigureAnalogChannels()
        Dim AnalogChan As Dataq.Devices.DI2108P.AnalogVoltageIn

        If cbCH1.Checked Then
            'configure analog channel 1
            AnalogChan = CType(TargetDevice.ChannelFactory(1, GetType(Dataq.Devices.DI2108P.AnalogVoltageIn)),
                Dataq.Devices.DI2108P.AnalogVoltageIn)
            AnalogChan.InputRange.Minimum = Text2Range(cmbGain1.Text, "Minimum")
            AnalogChan.InputRange.Maximum = Text2Range(cmbGain1.Text, "Maximum")
            AnalogChan.AcquisitionMode = Text2AcquisitionMode(cmbCH1.Text)
            AnalogChan.AcquisitionMode.Samples = CType(tbDecimation.Text, Integer)
        End If

        If cbCH2.Checked Then
            'configure analog channel 2
            AnalogChan = CType(TargetDevice.ChannelFactory(2, GetType(Dataq.Devices.DI2108P.AnalogVoltageIn)),
                Dataq.Devices.DI2108P.AnalogVoltageIn)
            AnalogChan.InputRange.Minimum = Text2Range(cmbGain2.Text, "Minimum")
            AnalogChan.InputRange.Maximum = Text2Range(cmbGain2.Text, "Maximum")
            AnalogChan.AcquisitionMode = Text2AcquisitionMode(cmbCH2.Text)
            AnalogChan.AcquisitionMode.Samples = CType(tbDecimation.Text, Integer)
        End If

        If cbCH3.Checked Then
            'configure analog channel 3
            AnalogChan = CType(TargetDevice.ChannelFactory(3, GetType(Dataq.Devices.DI2108P.AnalogVoltageIn)),
                Dataq.Devices.DI2108P.AnalogVoltageIn)
            AnalogChan.InputRange.Minimum = Text2Range(cmbGain3.Text, "Minimum")
            AnalogChan.InputRange.Maximum = Text2Range(cmbGain3.Text, "Maximum")
            AnalogChan.AcquisitionMode = Text2AcquisitionMode(cmbCH3.Text)
            AnalogChan.AcquisitionMode.Samples = CType(tbDecimation.Text, Integer)
        End If

        If cbCH4.Checked Then
            'configure analog channel 4
            AnalogChan = CType(TargetDevice.ChannelFactory(4, GetType(Dataq.Devices.DI2108P.AnalogVoltageIn)),
                Dataq.Devices.DI2108P.AnalogVoltageIn)
            AnalogChan.InputRange.Minimum = Text2Range(cmbGain4.Text, "Minimum")
            AnalogChan.InputRange.Maximum = Text2Range(cmbGain4.Text, "Maximum")
            AnalogChan.AcquisitionMode = Text2AcquisitionMode(cmbCH4.Text)
            AnalogChan.AcquisitionMode.Samples = CType(tbDecimation.Text, Integer)
        End If

        If cbCH5.Checked Then
            'configure analog channel 5
            AnalogChan = CType(TargetDevice.ChannelFactory(5, GetType(Dataq.Devices.DI2108P.AnalogVoltageIn)),
                Dataq.Devices.DI2108P.AnalogVoltageIn)
            AnalogChan.InputRange.Minimum = Text2Range(cmbGain5.Text, "Minimum")
            AnalogChan.InputRange.Maximum = Text2Range(cmbGain5.Text, "Maximum")
            AnalogChan.AcquisitionMode = Text2AcquisitionMode(cmbCH5.Text)
            AnalogChan.AcquisitionMode.Samples = CType(tbDecimation.Text, Integer)
        End If

        If cbCH6.Checked Then
            'configure analog channel 6
            AnalogChan = CType(TargetDevice.ChannelFactory(6, GetType(Dataq.Devices.DI2108P.AnalogVoltageIn)),
                Dataq.Devices.DI2108P.AnalogVoltageIn)
            AnalogChan.InputRange.Minimum = Text2Range(cmbGain6.Text, "Minimum")
            AnalogChan.InputRange.Maximum = Text2Range(cmbGain6.Text, "Maximum")
            AnalogChan.AcquisitionMode = Text2AcquisitionMode(cmbCH6.Text)
            AnalogChan.AcquisitionMode.Samples = CType(tbDecimation.Text, Integer)
        End If

        If cbCH7.Checked Then
            'configure analog channel 7
            AnalogChan = CType(TargetDevice.ChannelFactory(7, GetType(Dataq.Devices.DI2108P.AnalogVoltageIn)),
                Dataq.Devices.DI2108P.AnalogVoltageIn)
            AnalogChan.InputRange.Minimum = Text2Range(cmbGain7.Text, "Minimum")
            AnalogChan.InputRange.Maximum = Text2Range(cmbGain7.Text, "Maximum")
            AnalogChan.AcquisitionMode = Text2AcquisitionMode(cmbCH7.Text)
            AnalogChan.AcquisitionMode.Samples = CType(tbDecimation.Text, Integer)
        End If

        If cbCH8.Checked Then
            'configure analog channel 8
            AnalogChan = CType(TargetDevice.ChannelFactory(8, GetType(Dataq.Devices.DI2108P.AnalogVoltageIn)),
                Dataq.Devices.DI2108P.AnalogVoltageIn)
            AnalogChan.InputRange.Minimum = Text2Range(cmbGain8.Text, "Minimum")
            AnalogChan.InputRange.Maximum = Text2Range(cmbGain8.Text, "Maximum")
            AnalogChan.AcquisitionMode = Text2AcquisitionMode(cmbCH8.Text)
            AnalogChan.AcquisitionMode.Samples = CType(tbDecimation.Text, Integer)
        End If
    End Sub
    Private Function Text2Range(text As String, Boundary As String) As Double
        Select Case True
            Case text = "±2.5 V"
                If Boundary = "Minimum" Then
                    Return -2.5
                Else
                    Return 2.5
                End If
            Case text = "±5.0 V"
                If Boundary = "Minimum" Then
                    Return -5
                Else
                    Return 5
                End If
            Case text = "±10 V"
                If Boundary = "Minimum" Then
                    Return -10
                Else
                    Return 10
                End If
            Case text = "0-5 V"
                If Boundary = "Minimum" Then
                    Return 0
                Else
                    Return 5
                End If
            Case Else 'all that's left is the 0-10 V range
                If Boundary = "Minimum" Then
                    Return 0
                Else
                    Return 10
                End If
        End Select
    End Function

    Private Function Text2AcquisitionMode(text As String) As Dataq.Devices.IAcquisitionMode
        Select Case True
            Case text = "Average"
                Return New Dataq.Devices.DI2108P.AverageSample
            Case text = "Maximum"
                Return New Dataq.Devices.DI2108P.MaximumSample
            Case text = "Minimum"
                Return New Dataq.Devices.DI2108P.MinimumSample
        End Select
        Return New Dataq.Devices.DI2108P.LastSample
    End Function

    Private Async Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        cmbCH1.SelectedIndex = 0
        cmbCH2.SelectedIndex = 0
        cmbCH3.SelectedIndex = 0
        cmbCH4.SelectedIndex = 0
        cmbCH5.SelectedIndex = 0
        cmbCH6.SelectedIndex = 0
        cmbCH7.SelectedIndex = 0
        cmbCH8.SelectedIndex = 0
        cmbD0Mode.SelectedIndex = 0
        cmbD1Mode.SelectedIndex = 0
        cmbD2Mode.SelectedIndex = 0
        cmbD3Mode.SelectedIndex = 0
        cmbD4Mode.SelectedIndex = 0
        cmbD5Mode.SelectedIndex = 0
        cmbD6Mode.SelectedIndex = 0
        cmbDigOut0.SelectedIndex = 0
        cmbDigOut1.SelectedIndex = 0
        cmbDigOut2.SelectedIndex = 0
        cmbDigOut3.SelectedIndex = 0
        cmbDigOut4.SelectedIndex = 0
        cmbDigOut5.SelectedIndex = 0
        cmbDigOut6.SelectedIndex = 0
        cmbGain1.SelectedIndex = 0
        cmbGain2.SelectedIndex = 0
        cmbGain3.SelectedIndex = 0
        cmbGain4.SelectedIndex = 0
        cmbGain5.SelectedIndex = 0
        cmbGain6.SelectedIndex = 0
        cmbGain7.SelectedIndex = 0
        cmbGain8.SelectedIndex = 0

        ' Get a list of devices with model DI-2108
        Dim AllDevices As IReadOnlyList(Of Dataq.Devices.IDevice) =
            Await Dataq.Misc.Discovery.ByModel(GetType(Dataq.Devices.DI2108P.Device))
        If AllDevices.Count > 0 Then
            tbOutput.AppendText("Found a DI-2108." + Environment.NewLine)
            ' Cast first device from generic device to specific DI-2108 type
            TargetDevice = CType(AllDevices(0), Dataq.Devices.DI2108P.Device)
            ' Ensure it's stopped
            Await TargetDevice.AcquisitionStopAsync()
            ' Query device for some info
            Await TargetDevice.QueryDeviceAsync()
            ' Print queried info to GUI
            tbOutput.AppendText("Manufacturer: " + TargetDevice.Manufacturer + Environment.NewLine)
            tbOutput.AppendText("Model: " + TargetDevice.Model + Environment.NewLine)
            tbOutput.AppendText("Serial number: " + TargetDevice.Serial + Environment.NewLine)
            tbOutput.AppendText("Firmware revision: " + TargetDevice.FirmwareRevision.ToString() + Environment.NewLine)
            tbOutput.AppendText("Supported sample rates: " + TargetDevice.SupportedSampleRateRange.ToString() + Environment.NewLine)
            ' Enable the next usable button
        Else
            tbOutput.AppendText("No DI-2108 found." + Environment.NewLine)
        End If

        btnState.Enabled = True
    End Sub

    Private Async Sub btnDigOut_Click(sender As Object, e As EventArgs) Handles btnDigOut.Click
        'define a flag to indicate if at least one output port is defined
        Dim OutputExists As Boolean = False
        ' Store desired output states in array for easy access
        Dim DigOut(7) As Double
        DigOut(0) = Double.Parse(cmbDigOut0.Text)
        DigOut(1) = Double.Parse(cmbDigOut1.Text)
        DigOut(2) = Double.Parse(cmbDigOut2.Text)
        DigOut(3) = Double.Parse(cmbDigOut3.Text)
        DigOut(4) = Double.Parse(cmbDigOut4.Text)
        DigOut(5) = Double.Parse(cmbDigOut5.Text)
        DigOut(6) = Double.Parse(cmbDigOut6.Text)
        ' Loop through output channels
        For Each chan In TargetDevice.Channels
            If TypeOf chan Is Dataq.Devices.DI2108P.DigitalOut Then
                Dim temp As Dataq.Devices.DI2108P.DigitalOut =
                    CType(chan, Dataq.Devices.DI2108P.DigitalOut)
                ' Clear any previous data
                temp.DataOut.Clear()
                ' Add desired output value to channel
                temp.DataOut.Add(DigOut(temp.Number))
                OutputExists = True
            End If
        Next
        ' Ooutput data only if there's at least one enabled for output
        If OutputExists Then
            Await TargetDevice.SingleScanOutAsync()
        End If
    End Sub

    Private Sub cbD0_CheckedChanged(sender As Object, e As EventArgs) Handles cbD0.CheckedChanged
        cmbD0Mode.Enabled = cbD0.Checked
        cmbDigOut0.Enabled = cbD0.Checked And cmbD0Mode.Text = "Switch"
    End Sub

    Private Sub cbD1_CheckedChanged(sender As Object, e As EventArgs) Handles cbD1.CheckedChanged
        cmbD1Mode.Enabled = cbD1.Checked
        cmbDigOut1.Enabled = cbD1.Checked And cmbD1Mode.Text = "Switch"
    End Sub

    Private Sub cbD2_CheckedChanged(sender As Object, e As EventArgs) Handles cbD2.CheckedChanged
        cmbD2Mode.Enabled = cbD2.Checked
        cmbDigOut2.Enabled = cbD2.Checked And cmbD2Mode.Text = "Switch"
    End Sub

    Private Sub cbD3_CheckedChanged(sender As Object, e As EventArgs) Handles cbD3.CheckedChanged
        cmbD3Mode.Enabled = cbD3.Checked
        cmbDigOut3.Enabled = cbD3.Checked And cmbD3Mode.Text = "Switch"
    End Sub

    Private Sub cbD4_CheckedChanged(sender As Object, e As EventArgs) Handles cbD4.CheckedChanged
        cmbD4Mode.Enabled = cbD4.Checked
        cmbDigOut4.Enabled = cbD4.Checked And cmbD4Mode.Text = "Switch"
    End Sub

    Private Sub cbD5_CheckedChanged(sender As Object, e As EventArgs) Handles cbD5.CheckedChanged
        cmbD5Mode.Enabled = cbD5.Checked
        cmbDigOut5.Enabled = cbD5.Checked And cmbD5Mode.Text = "Switch"
    End Sub

    Private Sub cbD6_CheckedChanged(sender As Object, e As EventArgs) Handles cbD6.CheckedChanged
        cmbD6Mode.Enabled = cbD6.Checked
        cmbDigOut6.Enabled = cbD6.Checked And cmbD6Mode.Text = "Switch"
    End Sub

    Private Sub cbCH1_CheckedChanged(sender As Object, e As EventArgs) Handles cbCH1.CheckedChanged
        cmbCH1.Enabled = cbCH1.Checked
        cmbGain1.Enabled = cbCH1.Checked
    End Sub

    Private Sub cbCH2_CheckedChanged(sender As Object, e As EventArgs) Handles cbCH2.CheckedChanged
        cmbCH2.Enabled = cbCH2.Checked
        cmbGain2.Enabled = cbCH2.Checked
    End Sub

    Private Sub cbCH3_CheckedChanged(sender As Object, e As EventArgs) Handles cbCH3.CheckedChanged
        cmbCH3.Enabled = cbCH3.Checked
        cmbGain3.Enabled = cbCH3.Checked
    End Sub

    Private Sub cbCH4_CheckedChanged(sender As Object, e As EventArgs) Handles cbCH4.CheckedChanged
        cmbCH4.Enabled = cbCH4.Checked
        cmbGain4.Enabled = cbCH4.Checked
    End Sub

    Private Sub cbCH5_CheckedChanged(sender As Object, e As EventArgs) Handles cbCH5.CheckedChanged
        cmbCH5.Enabled = cbCH5.Checked
        cmbGain5.Enabled = cbCH5.Checked
    End Sub

    Private Sub cbCH6_CheckedChanged(sender As Object, e As EventArgs) Handles cbCH6.CheckedChanged
        cmbCH6.Enabled = cbCH6.Checked
        cmbGain6.Enabled = cbCH6.Checked
    End Sub

    Private Sub cbCH7_CheckedChanged(sender As Object, e As EventArgs) Handles cbCH7.CheckedChanged
        cmbCH7.Enabled = cbCH7.Checked
        cmbGain7.Enabled = cbCH7.Checked
    End Sub

    Private Sub cbCH8_CheckedChanged(sender As Object, e As EventArgs) Handles cbCH8.CheckedChanged
        cmbCH8.Enabled = cbCH8.Checked
        cmbGain8.Enabled = cbCH8.Checked
    End Sub

    Private Sub cmbD0Mode_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cmbD0Mode.SelectedIndexChanged
        cmbDigOut0.Enabled = cmbD0Mode.Text = "Switch"
    End Sub

    Private Sub cmbD1Mode_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cmbD1Mode.SelectedIndexChanged
        cmbDigOut1.Enabled = cmbD1Mode.Text = "Switch"
    End Sub

    Private Sub cmbD2Mode_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cmbD2Mode.SelectedIndexChanged
        cmbDigOut2.Enabled = cmbD2Mode.Text = "Switch"
    End Sub

    Private Sub cmbD3Mode_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cmbD3Mode.SelectedIndexChanged
        cmbDigOut3.Enabled = cmbD3Mode.Text = "Switch"
    End Sub

    Private Sub cmbD4Mode_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cmbD4Mode.SelectedIndexChanged
        cmbDigOut4.Enabled = cmbD4Mode.Text = "Switch"
    End Sub

    Private Sub cmbD5Mode_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cmbD5Mode.SelectedIndexChanged
        cmbDigOut5.Enabled = cmbD5Mode.Text = "Switch"
    End Sub

    Private Sub cmbD6Mode_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cmbD6Mode.SelectedIndexChanged
        cmbDigOut6.Enabled = cmbD6Mode.Text = "Switch"
    End Sub

    Private Async Sub ComboBox1_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cmbLED.SelectedIndexChanged
        Select Case True
            Case cmbLED.Text = "Black"
                Await TargetDevice.Protocol.SetLedColorAsync(0)
            Case cmbLED.Text = "Blue"
                Await TargetDevice.Protocol.SetLedColorAsync(1)
            Case cmbLED.Text = "Green"
                Await TargetDevice.Protocol.SetLedColorAsync(2)
            Case cmbLED.Text = "Cyan"
                Await TargetDevice.Protocol.SetLedColorAsync(3)
            Case cmbLED.Text = "Red"
                Await TargetDevice.Protocol.SetLedColorAsync(4)
            Case cmbLED.Text = "Magenta"
                Await TargetDevice.Protocol.SetLedColorAsync(5)
            Case cmbLED.Text = "Yellow"
                Await TargetDevice.Protocol.SetLedColorAsync(6)
            Case cmbLED.Text = "White"
                Await TargetDevice.Protocol.SetLedColorAsync(7)
        End Select
    End Sub
End Class
